package com.enbotsdk.en_bot_sdk

import android.graphics.Bitmap
import android.media.Image
import androidx.annotation.NonNull
import android.app.Activity
import android.content.res.AssetFileDescriptor
import android.graphics.BitmapFactory
import android.graphics.Typeface

import com.en.botsdk.Constants.LanguageAlignment
import com.en.botsdk.ui.ChatBotConfig
import io.flutter.FlutterInjector
import io.flutter.embedding.android.FlutterActivity

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** EnBotSdkPlugin */
class EnBotSdkPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private var activity: Activity? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "en_bot_sdk")
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "launchBot" -> {
          val botKey = call.argument<String>("botKey")
          val botName = call.argument<String>("botName")
          val brandingKey = call.argument<String>("brandingKey")
          val userId = call.argument<String>("userId")
          val chatHistorySize = call.argument<Int>("chatHistorySize")
          val showDone = call.argument<Boolean>("showDone")
          val languageDirection = call.argument<Int>("languageDirection")
          val headerTitleFont = call.argument<String>("headerTitleFont")
          val headerDescriptionFont = call.argument<String>("headerDescriptionFont")
          val sendButtonByteArray = call.argument<ByteArray>("sendMessageButtonIcon")
          var sendButtonBitmap: Bitmap? = null
          sendButtonByteArray?.let {
            sendButtonBitmap = BitmapFactory.decodeByteArray(
              sendButtonByteArray, 0,
            sendButtonByteArray.size
            )
          }

        val fontKeyTitle = FlutterInjector.instance().
        flutterLoader().getLookupKeyForAsset(headerTitleFont.orEmpty())
        val fontKeyDesc = FlutterInjector.instance().
        flutterLoader().getLookupKeyForAsset(headerDescriptionFont.orEmpty())
        val assetManager = activity?.assets
        val typeFaceTitle = Typeface.createFromAsset(assetManager, fontKeyTitle)
        val typeFaceDesc = Typeface.createFromAsset(assetManager, fontKeyDesc)

        val isRTL = if(languageDirection == 0) LanguageAlignment.DEFAULT else LanguageAlignment.RTL

        ChatBotConfig.getInstance().init(botKey, botName, true, brandingKey,
          activity, userId, chatHistorySize ?: 0, showDone ?: false, isRTL,
            typeFaceTitle, typeFaceDesc, 0, sendButtonBitmap)
        ChatBotConfig.getInstance().launchBot()
      }
      else -> result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
  }

  override fun onDetachedFromActivityForConfigChanges() {
    activity = null
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity
  }

  override fun onDetachedFromActivity() {
    activity = null
  }
}
